<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Magento\AdobeIms\Test\Unit\Model;

use Magento\AdobeIms\Model\GetToken;
use Magento\AdobeIms\Model\OAuth\TokenResponse;
use Magento\AdobeImsApi\Api\ConfigInterface;
use Magento\AdobeImsApi\Api\Data\TokenResponseInterfaceFactory;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\HTTP\Client\CurlFactory;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use Magento\Framework\UrlInterface;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

/**
 * Get user token test
 */
class GetTokenTest extends TestCase
{
    /**
     * @var ObjectManager
     */
    private $objectManager;

    /**
     * @var ConfigInterface|MockObject $config
     */
    private $configMock;

    /**
     * @var CurlFactory|MockObject $curlFactoryMock
     */
    private $curlFactoryMock;

    /**
     * @var Json|MockObject $jsonMock
     */
    private $jsonMock;

    /**
     * @var TokenResponseInterfaceFactory|MockObject $tokenResponseFactoryMock
     */
    private $tokenResponseFactoryMock;

    /**
     * @var GetToken $getToken
     */
    private $getToken;

    /**
     * @var UrlInterface|MockObject
     */
    private $url;

    /**
     * Prepare test objects.
     */
    protected function setUp(): void
    {
        $this->objectManager = new ObjectManager($this);
        $this->configMock = $this->getMockForAbstractClass(ConfigInterface::class);
        $this->curlFactoryMock = $this->createMock(CurlFactory::class);
        $this->jsonMock = $this->createMock(Json::class);
        $this->tokenResponseFactoryMock = $this->createMock(TokenResponseInterfaceFactory::class);
        $this->url = $this->getMockForAbstractClass(UrlInterface::class);
        $this->getToken = new GetToken(
            $this->configMock,
            $this->curlFactoryMock,
            $this->jsonMock,
            $this->tokenResponseFactoryMock,
            $this->url
        );
    }

    /**
     * Test save.
     */
    public function testExecute(): void
    {
        $curl = $this->createMock(Curl::class);
        $this->curlFactoryMock->expects($this->once())
            ->method('create')
            ->willReturn($curl);
        $curl->expects($this->exactly(2))
            ->method('addHeader')
            ->willReturn(null);
        $this->configMock->expects($this->once())
            ->method('getTokenUrl')
            ->willReturn('http://www.some.url.com');
        $this->configMock->expects($this->once())
            ->method('getApiKey')
            ->willReturn('string');
        $this->configMock->expects($this->once())
            ->method('getPrivateKey')
            ->willReturn('string');
        $curl->expects($this->once())
            ->method('post')
            ->willReturn(null);
        $this->jsonMock->expects($this->once())
            ->method('unserialize')
            ->willReturn(['string']);
        $tokenResponse = $this->createMock(TokenResponse::class);
        $this->tokenResponseFactoryMock->expects($this->once())
            ->method('create')
            ->willReturn($tokenResponse);
        $tokenResponse->expects($this->once())
            ->method('addData')
            ->willReturn($tokenResponse);
        $tokenResponse->expects($this->once())
            ->method('getAccessToken')
            ->willReturn('string');
        $tokenResponse->expects($this->once())
            ->method('getRefreshToken')
            ->willReturn('string');
        $this->assertEquals($tokenResponse, $this->getToken->execute('code'));
    }
}
